#include "panel.h"

/*
 * Abstract class representing a panel
 */

typedef struct
{
	AdwBin* content_bin;
	GtkBox* main_box;
	AdwBin* titlebar_bin;
	AdwHeaderBar* titlebar;

	GCancellable* cancellable;
	gboolean folded;
	gchar* title;
} PanelPrivate;

static void panel_buildable_init(GtkBuildableIface* iface);

G_DEFINE_ABSTRACT_TYPE_WITH_CODE(Panel, panel, ADW_TYPE_BIN,
                                 G_ADD_PRIVATE(Panel)
                                   G_IMPLEMENT_INTERFACE(GTK_TYPE_BUILDABLE, panel_buildable_init))

static GtkBuildableIface* parent_buildable_iface;

enum
{
	PROP_0,
	PROP_PARAMETERS,
	PROP_FOLDED,
	PROP_TITLE,
	N_PROPS
};

static GParamSpec* properties[N_PROPS];

static void
panel_buildable_add_child(GtkBuildable* buildable, GtkBuilder* builder, GObject* child, const char* type)
{
	PanelPrivate* priv = panel_get_instance_private(GINPUT_PANEL(buildable));

	if(GTK_IS_WIDGET(child) && !priv->main_box)
	{
		adw_bin_set_child(ADW_BIN(buildable), GTK_WIDGET(child));
		return;
	}

	if(g_strcmp0(type, "content") == 0)
		adw_bin_set_child(priv->content_bin, GTK_WIDGET(child));
	else if(g_strcmp0(type, "titlebar-start") == 0)
		adw_header_bar_pack_start(priv->titlebar, GTK_WIDGET(child));
	else if(g_strcmp0(type, "titlebar-end") == 0)
		adw_header_bar_pack_end(priv->titlebar, GTK_WIDGET(child));
	else if(g_strcmp0(type, "titlebar") == 0)
		adw_bin_set_child(priv->titlebar_bin, GTK_WIDGET(child));
	else
		parent_buildable_iface->add_child(buildable, builder, child, type);
}

/*
 * @Override from GtkBuildableIface
 */
static void
panel_buildable_init(GtkBuildableIface* iface)
{
	parent_buildable_iface = g_type_interface_peek_parent(iface);
	iface->add_child = panel_buildable_add_child;
}

/*
 * @Override from GObject
 */
static void
panel_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec)
{
	PanelPrivate* priv = panel_get_instance_private(GINPUT_PANEL(object));

	switch(prop_id)
	{
		case PROP_PARAMETERS:
		{
			g_autoptr(GVariant) v = NULL;
			GVariant* parameters;
			gsize n_parameters;

			parameters = g_value_get_variant(value);

			if(parameters == NULL)
				return;

			n_parameters = g_variant_n_children(parameters);
			if(n_parameters == 0)
				return;

			g_variant_get_child(parameters, 0, "v", &v);

			if(!g_variant_is_of_type(v, G_VARIANT_TYPE_DICTIONARY))
				g_warning("Wrong type for the first argument GVariant, expected 'a{sv}' but got '%s'",
				          (gchar*) g_variant_get_type(v));
			else if(g_variant_n_children(v) > 0)
				g_warning("Ignoring additional flags");

			if(n_parameters > 1)
				g_warning("Ignoring additional parameters");

			break;
		}

		case PROP_TITLE:
			priv->title = g_value_dup_string(value);
			break;

		case PROP_FOLDED:
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
}

/*
 * @Override from GObject
 */
static void
panel_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* pspec)
{
	PanelPrivate* priv = panel_get_instance_private(GINPUT_PANEL(object));

	switch(prop_id)
	{
		case PROP_FOLDED:
			g_value_set_boolean(value, priv->folded);
			break;

		case PROP_TITLE:
			g_value_set_string(value, priv->title);
			break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
			break;
	}
}

/*
 * @Override from GObject
 */
static void
panel_finalize(GObject* object)
{
	PanelPrivate* priv = panel_get_instance_private(GINPUT_PANEL(object));

	g_cancellable_cancel(priv->cancellable);
	g_clear_object(&priv->cancellable);

	g_clear_pointer(&priv->title, g_free);

	G_OBJECT_CLASS(panel_parent_class)->finalize(object);
}

static void
panel_class_init(PanelClass* klass)
{
	GObjectClass* object_class = G_OBJECT_CLASS(klass);
	GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(klass);

	object_class->get_property = panel_get_property;
	object_class->set_property = panel_set_property;
	object_class->finalize = panel_finalize;

	//   signals[SIDEBAR_ACTIVATED] = g_signal_new ("sidebar-activated",
	//                                              G_TYPE_FROM_CLASS (object_class),
	//                                              G_SIGNAL_RUN_LAST,
	//                                              0, NULL, NULL,
	//                                              g_cclosure_marshal_VOID__VOID,
	//                                              G_TYPE_NONE, 0);

	properties[PROP_FOLDED] = g_param_spec_boolean("folded", NULL, NULL,
	                                               FALSE,
	                                               G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);

	properties[PROP_PARAMETERS] = g_param_spec_variant("parameters",
	                                                   "Structured parameters",
	                                                   "Additional parameters passed externally (ie. command line, D-Bus activation)",
	                                                   G_VARIANT_TYPE("av"),
	                                                   NULL,
	                                                   G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);

	properties[PROP_TITLE] = g_param_spec_string("title", NULL, NULL, NULL,
	                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);

	g_object_class_install_properties(object_class, N_PROPS, properties);

	gtk_widget_class_set_template_from_resource(widget_class, "/v/ginput/panel.ui");

	gtk_widget_class_bind_template_child_private(widget_class, Panel, content_bin);
	gtk_widget_class_bind_template_child_private(widget_class, Panel, main_box);
	gtk_widget_class_bind_template_child_private(widget_class, Panel, titlebar_bin);
	gtk_widget_class_bind_template_child_private(widget_class, Panel, titlebar);
}

static void
panel_init(Panel* panel)
{
	gtk_widget_init_template(GTK_WIDGET(panel));
}
